
# THIS FILE IS GENERATED FROM MDTRAJ SETUP.PY
short_version = '0.6.1'
version = '0.6.1'
full_version = '0.6.1'
git_revision = 'ecacd1c118e6eec0492322bcd95f7910b22fed09'
release = True

if not release:
    version = full_version
